// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#include "version.h"

#define APSTUDIO_READONLY_SYMBOLS
///////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "windows.h"

///////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS


#define QUOTE(str) #str
#define EXPAND_AND_QUOTE(str) QUOTE(str)

///////////////////////////////////
// Version
1 VERSIONINFO
FILEVERSION CASPAR_GEN, CASPAR_MAJOR, CASPAR_MINOR
PRODUCTVERSION CASPAR_GEN, CASPAR_MAJOR, CASPAR_MINOR
FILEFLAGSMASK 0
FILEOS 0x40000
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
   VALUE "CompanyName", "SVT"
   VALUE "FileDescription", "CasparCG video and graphics play-out server."
   VALUE "FileVersion", EXPAND_AND_QUOTE(CASPAR_GEN) "." EXPAND_AND_QUOTE(CASPAR_MAJOR) "." EXPAND_AND_QUOTE(CASPAR_MINOR) " " CASPAR_HASH
   VALUE "InternalName", ""
   VALUE "LegalCopyright", "GPLv3"
   VALUE "OriginalFilename", "casparcg.exe"
   VALUE "ProductName", "CasparCG"
   VALUE "ProductVersion", EXPAND_AND_QUOTE(CASPAR_GEN) "." EXPAND_AND_QUOTE(CASPAR_MAJOR) "." EXPAND_AND_QUOTE(CASPAR_MINOR) " " CASPAR_HASH
 END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END

///////////////////////////////////
// Swedish (Sweden) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_SVE)
LANGUAGE LANG_SWEDISH, SUBLANG_SWEDISH

#ifdef APSTUDIO_INVOKED
///////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


///////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
MAINICON            ICON                    "CasparCG.ico"
#endif    // Swedish (Sweden) resources
///////////////////////////////////



#ifndef APSTUDIO_INVOKED
///////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


///////////////////////////////////
#endif    // not APSTUDIO_INVOKED
